
# q-expansion of newform 3024.2.cz.b, downloaded from the LMFDB on 28 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3024
weight = 2
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[1135, [-1, 0]], [757, [1, 0]], [785, [0, -1]], [2593, [0, 1]]]
aps_data = [[0, 0], [0, 0], [0, 0], [2, -3], [-4, 2], [-2, 1], [-1, -1], [0, 5], [2, 2], [3, -3], [-1, 0], [0, -7], [2, -1], [-1, -1], [-9, 0], [-9, 9], [-15, 0], [1, -2], [-9, 18], [6, -12], [1, 1], [-1, 2], [-9, 9], [-2, 1], [-1, -1], [16, -8], [-8, 8], [-2, 1], [-11, 11], [0, -9], [-6, 12], [-12, 12], [0, -18], [0, -5], [-6, 6], [-20, 10], [7, -14], [18, -9], [0, 21], [11, -22], [-7, -7], [-8, 16], [1, -2], [-17, 0], [-6, 0], [-1, 1], [-14, 7], [5, -5], [0, -12], [8, 8], [0, -21], [-26, 13], [-24, 12], [0, 0], [16, 16], [36, -18], [-13, -13], [0, 1], [0, 14], [15, -15], [31, 0], [-30, 15], [-8, 0], [-3, 0], [17, -34], [21, 0], [3, -6], [0, -11], [13, -26], [15, 15], [24, -12], [14, -7], [0, 8], [10, -10], [14, -28], [-24, 24], [0, 18], [18, -9], [30, -30], [-17, 34], [0, -15], [5, -5], [-15, -15], [8, -16], [25, 0], [-15, 30], [-18, 0], [-1, 0], [13, 13], [42, -21], [0, 15], [0, 0], [-9, -9], [-10, 5], [-2, -2], [-24, 0], [20, 20], [-13, -13], [0, 13], [0, -43], [-5, -5], [-33, 33], [21, 0], [-3, 0], [23, -46], [-15, -15], [27, -27], [-10, 5], [21, -42], [-21, -21], [32, -32], [-35, 35], [0, 27], [0, 28], [18, -36], [0, 6], [0, -13], [-15, 15], [30, -30], [13, 13], [-5, 10], [-7, 7], [-1, 2], [25, 25], [5, 0], [-18, 0], [-37, 0], [0, 27], [-35, 35], [12, 12], [-9, -9], [14, -7], [10, 10], [2, 0], [28, 28], [6, -3], [19, 19], [-13, 0], [-38, 19], [-9, 9], [-52, 0], [33, 0], [7, -14], [10, -20], [9, 9], [27, -27], [31, 31], [8, -4], [-4, 4], [30, -15], [46, -46], [12, -24], [-18, 36], [12, -12], [-44, 22], [33, 33], [26, -13], [-5, 10], [-28, 56], [17, -34], [-3, 6], [18, 0], [18, -9], [0, 15], [-51, 0], [0, 12], [-21, -21], [64, -32], [-50, 0], [24, 24], [-68, 34], [-42, 21], [6, 6], [0, 26], [-5, 0], [-14, 7], [3, 3], [-9, 9], [56, 0], [-11, 22], [-1, 2], [39, -39], [0, 5], [30, -15], [-19, 38], [-72, 36], [-11, 11], [0, -44], [50, -50], [-3, 3], [42, -21], [-15, -15], [-6, 3], [3, -6], [1, 1], [-23, 46], [-5, 0], [-17, 0], [29, 29], [0, -9], [-39, 0], [-31, 31], [-8, -8], [-32, 16], [12, 0], [-17, -17], [0, 29], [30, 30], [3, -3], [-17, 0], [0, -47], [34, -17], [-29, -29], [-69, 0], [21, 0], [33, -66], [7, -14], [72, -36], [-58, 29], [0, -69], [-50, 50], [0, 31], [30, -30], [-14, 7], [0, -27], [53, -53], [-9, 18], [-19, -19], [5, 0], [-7, 14], [23, 0], [11, -11], [-14, 7], [-3, 0], [-19, 19], [0, -48], [-36, -36], [0, 39], [-26, 13], [-72, 0], [-68, 34], [0, 73], [-65, 0], [0, -43], [-30, 15], [-69, 0], [8, 0], [57, 0], [-17, 34], [14, -28], [0, -23], [-34, 17], [-35, 70], [15, 15], [-40, 20], [-26, 13], [5, -5], [0, -28], [8, -16], [74, -37], [60, -30], [-27, -27], [15, -30], [-32, 64], [-15, 30], [-55, 0], [78, 0], [1, 1], [-86, 43], [-39, 0], [-8, -8], [-9, -9], [24, -12], [-42, -42], [62, -31], [0, 37], [-26, -26], [0, -34], [0, 53], [-57, 0], [33, 0], [39, -78], [39, -39], [-9, -9], [4, -4], [6, -3], [-83, 83], [0, -21], [0, 64], [-12, 24], [0, -30], [-27, 27], [6, -6], [0, -63], [1, -1], [25, -50], [-35, 0], [-27, -27], [0, 15], [-35, 35], [28, 28], [0, -69], [11, 11]]
